/*
 * Copyright (c) 2018-2028, aokunsang All rights reserved.
 * 我问青山何时老，青山问我几时闲，
 * 不是闲人闲不得，能闲必非等闲人。
 */
package org.springblade.visual.enums;

import java.math.BigDecimal;
import java.math.RoundingMode;

import lombok.AllArgsConstructor;
import lombok.Getter;

/**
 * 同环比：数据设置
 *
 * @author wookvn
 * @datetime 2024-10-15 17:30:57
 */
@Getter
@AllArgsConstructor
public enum CompareDataTypeEnum {

    /**
     *  具体数值：显示上一期的具体数值
     */
    EXACT {
        @Override
        public BigDecimal apply(BigDecimal curData, BigDecimal preData, CompareRatioCalcTypeEnum ratioCalcType) {
            return preData != null ? preData.setScale(CompareDataTypeEnum.RADIX_POINT, RoundingMode.HALF_UP) : null;
        }
    },

    /**
     * 对比差值： 本期值-上期值
     */
    DIFF {
        @Override
        public BigDecimal apply(BigDecimal curData, BigDecimal preData, CompareRatioCalcTypeEnum ratioCalcType) {
            return curData != null && preData != null ? curData.subtract(preData).setScale(CompareDataTypeEnum.RADIX_POINT, RoundingMode.HALF_UP) : null;
        }
    },

    /**
     * 差值百分比：（本期值-上期值）/上期值 *100%
     */
    PERCENT {
        @Override
        public BigDecimal apply(BigDecimal curData, BigDecimal preData, CompareRatioCalcTypeEnum ratioCalcType) {
            if (curData == null || preData == null || preData.compareTo(BigDecimal.ZERO) == 0) {
                return null;
            }
            //仅当上期数据 < 0时，使用计算公式
            if(preData.compareTo(BigDecimal.ZERO) < 0 && ratioCalcType != null) {
                return ratioCalcType.apply(curData, preData);
            }
            return curData.subtract(preData).divide(preData, CompareDataTypeEnum.RADIX_POINT, RoundingMode.HALF_UP);
        }
    },

    /**
     * 比值百分比：（本期值/上期值）*100%
     */
    PROPORTION {
        @Override
        public BigDecimal apply(BigDecimal curData, BigDecimal preData, CompareRatioCalcTypeEnum ratioCalcType) {
            return curData != null && preData != null && preData.compareTo(BigDecimal.ZERO) != 0
                    ? curData.divide(preData, CompareDataTypeEnum.RADIX_POINT, RoundingMode.HALF_UP) : null;
        }
    },
    ;

    /**
     * 小数点精确位数
     */
    public final static int RADIX_POINT = 8;

    /**
     * <p>计算方式</p>
     * @author wookvn
     * @datetime 2024-10-16 17:32:01
     * @param curData: 上期数据
     * @param preData: 下期数据
     * @return java.math.BigDecimal
     **/
    public abstract BigDecimal apply(BigDecimal curData, BigDecimal preData, CompareRatioCalcTypeEnum ratioCalcType);
}
