/**
 * 
 */
package org.springblade.visual.service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.function.Consumer;
import java.util.stream.Collectors;

import cn.hutool.core.map.MapUtil;
import com.google.common.collect.Maps;
import org.springblade.core.log.properties.PlatomixLogProperties;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.core.tool.exception.ServiceException;
import org.springblade.core.tool.utils.CollectionUtil;
import org.springblade.cube.enums.DimensionTypeEnum;
import org.springblade.cube.model.CubeData;
import org.springblade.cube.service.CubeDataService;
import org.springblade.visual.dto.ComponentColumn;
import org.springblade.visual.dto.ComponentColumnCompare;
import org.springblade.visual.dto.ComponentDto;
import org.springblade.visual.dto.ComponentExtJson;
import org.springblade.visual.dto.ComponentFilterCondition;
import org.springblade.visual.dto.QuerySqlParam;
import org.springblade.visual.entity.VisualComponent;
import org.springblade.visual.enums.ComponentUseTypeEnum;
import org.springblade.visual.mapper.VisualComponentMapper;
import org.springblade.visual.service.component.VisualComponentCompareService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.microsoft.sqlserver.jdbc.StringUtils;

import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * @author wookvn
 *
 */
@Slf4j
@Service
public class VisualComponentService extends ServiceImpl<VisualComponentMapper, VisualComponent> {

	@Autowired
	private QuerySqlService querySqlService;

	@Autowired
	private CubeDataService cubeDataService;

	@Autowired
	private PlatomixLogProperties logProperties;

	@Autowired
	private VisualDataPermissionService visualDataPermissionService;

	@Autowired
	private VisualComponentCompareService visualComponentCompareService;

	/**
	 * @param visualId
	 * @param componentDtoList
	 */
	@Transactional(rollbackFor = ServiceException.class)
	public void batchSaveComponent(Integer visualId, List<ComponentDto> componentDtoList, BladeUser bladeUser) {
		for(ComponentDto componentDto : componentDtoList) {
			VisualComponent component = componentDto.getComponent();
			//新增
			if(component.getId() == null) {
				component.setCreateTime(new Date());
				component.setCreateBy(SecureUtil.getNickName());
				component.setCreateUserId(SecureUtil.getUserId());
			}
			component.setVisualId(visualId);
			component.setColumnJson(componentDto.getColumnList());
			component.setFilterJson(componentDto.getFilterList());
			component.setFilterRelationJson(componentDto.getFilterRelation());
			component.setSortJson(componentDto.getSortList());
			 //当且仅当组件类型是Cube并且有字段信息解析SQL（有可能是个纯前端的组件或者是API类型组件）
			if(!StrUtil.isEmpty(componentDto.getColumnList())
					&& component.getUseType() == ComponentUseTypeEnum.CUBE) {
				LambdaQueryWrapper<CubeData> queryWrapper = Wrappers.<CubeData>lambdaQuery().select(CubeData::getDatasourceId).eq(CubeData::getId, component.getCubeDataId());
				Integer datasourceId = cubeDataService.getObj(queryWrapper, t -> Integer.valueOf(t.toString()));
				//设置数据源ID
				component.setDatasourceId(datasourceId);

				//解析使用到的字段ID列表
				Set<Integer> usedColumnIdList = querySqlService.parseUsedColumnIdList(componentDto);
				//如果extJson中存在zuanList字段，也需要放到columnId列表中
				if(StrUtil.isNotBlank(component.getExtJson())) {
					try {
						JSONObject tempObject = JSON.parseObject(component.getExtJson());
						if(tempObject.containsKey("zuanList")) {
							JSON.parseArray(tempObject.getString("zuanList"), ComponentColumn.class).stream().map(ComponentColumn::getCubeColumnId).forEach(usedColumnIdList::add);;
						}
					} catch (Exception e) {
						log.error(e.getMessage(), e);
					}
				}
				//设置使用到的字段ID列表JSON
				component.setColumnIdList(JSON.toJSONString(usedColumnIdList));
			}
			this.saveOrUpdate(component);
		}
	}
	/**
	 * 显示sql的方式
	 *
	 * @param sql
	 * @return
	 */
	public String showSql(String sql) {
		if (logProperties.getApiDetailInfoEnabled()) {
			return logProperties.getApiDetailInfoBase64Enabled() ? cn.hutool.core.codec.Base64.encode(sql) : sql;
		}
		return null;
	}

	/**
	 * <p>组件预览前校验，并返回json反序列化后对象</p>
	 * @author wookvn
	 * @datetime 2024-10-12 11:57:28
	 * @param base64Value: base64参数值
	 * @return org.springblade.visual.dto.ComponentDto
	 **/
	public ComponentDto previewCheck(String base64Value) {
		return previewCheck(base64Value, false, null);
	}

	/**
	 * <p>组件预览前校验，并返回json反序列化后对象</p>
	 * @author wookvn
	 * @datetime 2024-10-12 11:57:28
	 * @param base64Value: base64参数值
	 * @param extraValidateConsumer: 额外的校验回调函数
	 * @return org.springblade.visual.dto.ComponentDto
	 **/
	public ComponentDto previewCheck(String base64Value, boolean checkDateComplete, Consumer<ComponentDto> extraValidateConsumer) {
		String decoderStr = cn.hutool.core.codec.Base64.decodeStr(base64Value);
		ComponentDto componentDto = JSON.parseObject(decoderStr, ComponentDto.class);

		// 判断当前的查询是否需要补齐数据
		if(checkDateComplete) {
			String filterColum = getFilterColum(componentDto.getColumnList());
			if (StrUtil.isNotEmpty(filterColum)) {
				List<Object> selectTimes = getSelectTime(componentDto.getFilterList(), filterColum);
				if (selectTimes.size() < 1) {
					throw new ServiceException("请设置查询的时间范围...");
				}
				componentDto.setIsDateComplete(true);
				componentDto.setFilterColum(filterColum);
				componentDto.setSelectTime(selectTimes);
			}
		}

		if (StringUtils.isEmpty(componentDto.getColumnList())) {
			throw new ServiceException("字段列表不能为空");
		}
		if (componentDto.getVisualDataId() != null && componentDto.getVisualDataCreateUserId() != null) {
			//校验大屏是否有访问权限
			if (!visualDataPermissionService.checkVisualDataForUser(componentDto.getVisualDataId(), componentDto.getVisualDataCreateUserId(), AuthUtil.getUser())) {
				throw new ServiceException("该大屏没有访问权限");
			}
		}
		VisualComponent component = componentDto.getComponent();
		if (component == null || component.getCubeDataId() == null) {
			throw new ServiceException("数据集为空");
		}
		//额外的校验
		if(extraValidateConsumer != null) {
			extraValidateConsumer.accept(componentDto);
		}
		return componentDto;
	}

	public String getFilterColum(String columnLists) {
		List<ComponentColumn> columnList = JSON.parseArray(columnLists, ComponentColumn.class);
		if (columnList != null && columnList.size() > 0) {
			for (ComponentColumn componentColumn : columnList) {
				ComponentExtJson componentExtJson = JSON.parseObject(componentColumn.getExtJson(), ComponentExtJson.class);
				if (componentExtJson.getIsDateComplete() != null && componentExtJson.getIsDateComplete()) {
					return componentColumn.getColumnName();
				}
			}
		}
		return null;
	}

	public List<Object> getSelectTime(String filterList, String filterColum) {
		JSONArray jsonArray = JSONUtil.parseArray(filterList);
		List<Object> selectTime = new ArrayList<>();
		for (Object obj : jsonArray) {
			cn.hutool.json.JSONObject entries = JSONUtil.parseObj(obj);
			ComponentColumn column = JSON.parseObject(entries.getStr("column"), ComponentColumn.class);
			if (column.getColumnName().equals(filterColum)) {
				String conditions = entries.getStr("conditions");
				List<ComponentFilterCondition> filterConditions = JSON.parseArray(conditions, ComponentFilterCondition.class);
				if (filterConditions != null && filterConditions.size() > 0) {
					List<Object> selectTimes = new ArrayList<>();
					for (ComponentFilterCondition filterCondition : filterConditions) {
						selectTimes.add(filterCondition.getValues());
					}
					List<Object> objs = (List<Object>) selectTimes.get(0);
					if (objs != null & objs.size() >= 1) {
						selectTime.clear();
						selectTime.addAll(selectTimes);
					}
				}
			}
		}
		return selectTime;
	}


	/**
	 * <p>预览数据后置处理</p>
	 * @author wookvn
	 * @datetime 2024-10-16 18:47:12
	 * @param fieldDict: 字段别名的字典映射
	 * @param resultList: 数据结果
	 * @param columnList: 字段列表
	 * @param excludeColumnIdList: 无权限的列ID
	 * @param querySqlParam：查询参数
	 **/
	public void postPreviewData(Map<String, String> fieldDict, List<Map<String, Object>> resultList, List<ComponentColumn> columnList, Set<Integer> excludeColumnIdList, QuerySqlParam querySqlParam) {
		//列权限，删除掉不显示的列
//		if (CollectionUtil.isNotEmpty(excludeColumnIdList) && CollectionUtil.isNotEmpty(resultList)) {
//			Map<Integer, String> fieldIdMap = columnList.stream().collect(Collectors.toMap(ComponentColumn::getCubeColumnId, ComponentColumn::getColumnAliasName));
//			List<String> excludeColumnAliasList = excludeColumnIdList.stream().map(it -> fieldIdMap.get(it)).filter(Objects::nonNull).collect(Collectors.toList());
//			if (CollectionUtil.isNotEmpty(excludeColumnAliasList)) {
//				//1.格式化数据
//				for (Map<String, Object> jsonObj : resultList) {
//					excludeColumnAliasList.stream().forEach(item -> jsonObj.remove(item));
//				}
//				//2.格式化fieldDict
//				if (fieldDict != null) {
//					excludeColumnAliasList.stream().forEach(item -> fieldDict.remove(item));
//				}
//			}
//		}
//		// 同环比的后置处理
//		visualComponentCompareService.afterCompareHandle(fieldDict, resultList, columnList);

		if(CollectionUtil.isNotEmpty(resultList)) {
			//(1.1)列权限，删除掉不显示的列
			List<String> excludeColumnAliasList = new ArrayList<>();
			if (CollectionUtil.isNotEmpty(excludeColumnIdList)) {
				Map<Integer, String> fieldIdMap = columnList.stream().collect(Collectors.toMap(ComponentColumn::getCubeColumnId, ComponentColumn::getColumnAliasName));
				excludeColumnIdList.stream().map(it -> fieldIdMap.get(it)).filter(Objects::nonNull).forEach(excludeColumnAliasList::add);
			}

			//(1.2)获取需要处理同环比的字段
			List<ComponentColumn> compareColumnList = columnList.stream().filter(item -> item.getDimensionType() == DimensionTypeEnum.ZHIBIAO && item.getCompare() != null
							&& StrUtil.isNotBlank(item.getCompare().getDateFieldUniqId())).collect(Collectors.toList());
			compareColumnList.stream().map(item -> item.getColumnAliasName() + VisualComponentCompareService.LAG_SYMBOL).forEach(excludeColumnAliasList::add);

			int startDate = 0, endDate = 0;
			String timeColumn = querySqlParam.getTimeColum();
			//(1.3)日期补全后置处理
			if (StrUtil.isNotEmpty(timeColumn) &&
					CollectionUtil.isNotEmpty(querySqlParam.getSelectTime()) && CollectionUtil.isNotEmpty(querySqlParam.getHourTime())) {
				List<String> selectTime = querySqlParam.getSelectTime();
				List<String> hourTime = querySqlParam.getHourTime();
				String endTime = selectTime.get(1);
				String startTime = selectTime.get(0);
				String startHour = hourTime.get(0);
				String endHour = hourTime.get(1);
				startDate = Integer.parseInt(startTime + startHour);
				endDate = Integer.parseInt(endTime + endHour);
				if (startHour.length() < 2) {
					startDate = Integer.parseInt(startTime + 0 + startHour);
				}
				if (endHour.length() < 2) {
					endDate = Integer.parseInt(endTime + 0 + endHour);
				}
			}

			//2.1.处理fieldDict
			if (fieldDict != null && !excludeColumnAliasList.isEmpty()) {
				excludeColumnAliasList.stream().forEach(fieldDict::remove);
			}

			//2.2.处理数据
			for(Iterator<Map<String, Object>> it = resultList.iterator(); it.hasNext();) {
				Map<String, Object> jsonObj = it.next();
				//(1.1)处理日期补全
				if (StrUtil.isNotEmpty(timeColumn)) {
					Object object = jsonObj.get(timeColumn);
					if (object != null) {
						int date = Integer.parseInt(object.toString());
						//删除该记录
						if (date < startDate || date > endDate) {
							it.remove();
							continue;
						}
					}
				}
				//(1.2)处理同环比的数据
				if (!compareColumnList.isEmpty()) {
					for (ComponentColumn column : compareColumnList) {
						String curAliasColumnName = column.getColumnAliasName(), preAliasColumnName = curAliasColumnName + VisualComponentCompareService.LAG_SYMBOL;
						Object _curData = jsonObj.get(curAliasColumnName), _preData = jsonObj.get(preAliasColumnName);
						BigDecimal curData = _curData != null ? new BigDecimal(_curData.toString()) : null,
								   preData = _preData != null ? new BigDecimal(_preData.toString()) : null;
						ComponentColumnCompare columnCompare = column.getCompare();
						BigDecimal resultData = columnCompare.getCompareDataType().apply(curData, preData, columnCompare.getCompareRatioCalcType());
						//删除上一期的数据
						jsonObj.remove(preAliasColumnName);
						//更新本期字段为结果值
						jsonObj.put(curAliasColumnName, resultData);
					}
				}
			}
		}
	}

	public static void main(String[] args) {
		List<Map<String, Object>> list = new ArrayList<>();
		list.add(MapUtil.of("a", 100));
		list.add(MapUtil.of("b", 200));
		list.add(MapUtil.of("c", 300));
		for(Iterator<Map<String, Object>> it = list.iterator(); it.hasNext();) {
			Map<String, Object> jsonObj = it.next();
			jsonObj.put("a", "111");
		}
		System.out.println(JSON.toJSONString(list));
	}
}
