package org.springblade.visual.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springblade.core.mp.base.TenantEntity;
import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.core.tool.exception.ServiceException;
import org.springblade.core.tool.utils.CollectionUtil;
import org.springblade.system.client.TenantClient;
import org.springblade.system.client.UserClient;
import org.springblade.system.client.model.TenantModel;
import org.springblade.system.client.model.UserModel;
import org.springblade.visual.dto.CreateUserNameDto;
import org.springblade.visual.dto.EnableParams;
import org.springblade.visual.dto.PageComponentTemplateParams;
import org.springblade.visual.dto.ReviewComponentTemplateParams;
import org.springblade.visual.entity.VisualComponentTemplate;
import org.springblade.visual.entity.VisualComponentTemplateCategory;
import org.springblade.visual.entity.VisualReviewLog;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class VisualComponentTemplateManageService {

    @Autowired
    private VisualComponentTemplateCategoryService visualComponentTemplateCategoryService;

    @Autowired
    private VisualComponentTemplateService visualComponentTemplateService;
    @Autowired
    private VisualReviewLogService visualReviewLogService;
    @Autowired
    private UserClient userClient;
    @Autowired
    private TenantClient tenantClient;

    /**
     * 创建目录
     */
    public boolean submitCategory(VisualComponentTemplateCategory category) {
        if (category.getId() == null) {
            visualComponentTemplateCategoryService.createCategory(category);
        } else {
            visualComponentTemplateCategoryService.editCategory(category);
        }
        return true;
    }

    /**
     * 删除分类
     */
    public boolean deleteCategory(Long id) {
        //判断分类下是否有组件模板
        List<VisualComponentTemplate> list1 = visualComponentTemplateService.list(new QueryWrapper<VisualComponentTemplate>().eq("mcp_category", id));
        if (list1.size() > 0) throw new ServiceException("分类下有组件模板，请勿删除");
        //判断是否有子分类
        List<VisualComponentTemplateCategory> list = visualComponentTemplateCategoryService.list(new QueryWrapper<VisualComponentTemplateCategory>().eq("parent_id", id));
        if (list.size() > 0) throw new ServiceException("请先删除子分类");
        return visualComponentTemplateCategoryService.deleteCategory(id);
    }

    /**
     * 查询所有目录(租户id是null的数据)
     */
    public List<VisualComponentTemplateCategory> queryAllCategory(String name) {
        return visualComponentTemplateCategoryService.queryMcpAllCategory(name);
    }

    /**
     * 组件模板列表
     */
    public IPage<VisualComponentTemplate> pageComponentTemplate(PageComponentTemplateParams params) {
        LambdaUpdateWrapper<VisualComponentTemplate> wrapper = Wrappers.<VisualComponentTemplate>lambdaUpdate()
                .eq(params.getCategory() != null, VisualComponentTemplate::getMcpCategory, params.getCategory())
                .like(params.getName() != null, VisualComponentTemplate::getName, params.getName())
                .eq(params.getCreateUser() != null, VisualComponentTemplate::getCreateUser, params.getCreateUser())
                .eq(params.getReviewStatus() != null, VisualComponentTemplate::getReviewStatus, params.getReviewStatus())
                .eq(params.getTenantId() != null, TenantEntity::getTenantId, params.getTenantId())
                .ne(VisualComponentTemplate::getReviewStatus, 0)
                .eq(params.getStatus() != null, VisualComponentTemplate::getStatus, params.getStatus())
                .between(params.getStartTime() != null && params.getEndTime() != null, VisualComponentTemplate::getCreateTime, params.getStartTime()+" 00:00:00", params.getEndTime()+" 23:59:59")
                .orderByDesc(VisualComponentTemplate::getCreateTime);
        Page<VisualComponentTemplate> page = visualComponentTemplateService.page(new Page<>(params.getPage(), params.getSize()), wrapper);
        if (page.getRecords().size() != 0) {
            List<String> tenantIds = page.getRecords().stream().map(TenantEntity::getTenantId).collect(Collectors.toList());
            List<TenantModel> tenantByTenantIds = tenantClient.getTenantByTenantIds(tenantIds);
            Map<String, String> tenantMap = tenantByTenantIds.stream().collect(Collectors.toMap(TenantModel::getTenantId, TenantModel::getTenantName));
            //获取分类信息
            Map<Integer, String> categoryMap = new HashMap<>();
            List<Long> mcpCategoryIds = page.getRecords().stream().map(VisualComponentTemplate::getMcpCategory).collect(Collectors.toList());
            if (mcpCategoryIds.size() != 0) {
                List<VisualComponentTemplateCategory> visualComponentTemplateCategories = visualComponentTemplateCategoryService.listByIds(mcpCategoryIds);
                categoryMap = visualComponentTemplateCategories.stream().collect(Collectors.toMap(VisualComponentTemplateCategory::getId, VisualComponentTemplateCategory::getName));
            }
            List<Long> userIds = page.getRecords().stream().map(VisualComponentTemplate::getCreateUser).collect(Collectors.toList());
            List<UserModel> userModelList = userClient.listByIds(userIds);
            Map<Long, String> accountMap = new HashMap<>();
            if (CollectionUtil.isNotEmpty(userModelList)) {
                accountMap = userModelList.stream().collect(Collectors.toMap(UserModel::getId, UserModel::getAccount));
            }
            for (VisualComponentTemplate visualComponentTemplate : page.getRecords()) {
                visualComponentTemplate.setTenantName(tenantMap.get(visualComponentTemplate.getTenantId()));
                if (categoryMap.size() != 0) {
                    visualComponentTemplate.setMcpCategoryName(categoryMap.get(visualComponentTemplate.getMcpCategory()));
                    visualComponentTemplate.setIamAccount(accountMap.get(visualComponentTemplate.getCreateUser()));
                }
            }
        }
        return page;
    }

    @Transactional(rollbackFor = Exception.class)
    public boolean reviewTemplate(ReviewComponentTemplateParams params) {
        visualComponentTemplateService.update(
                Wrappers.<VisualComponentTemplate>lambdaUpdate()
                        .eq(VisualComponentTemplate::getId, params.getId())
                        .set(VisualComponentTemplate::getStatus,params.isPass()?1:0)
                        .set(VisualComponentTemplate::getReviewStatus, params.isPass() ? 2 : 3)
                        .set(VisualComponentTemplate::getMcpCategory, params.getCategory())
        );
        VisualReviewLog visualReviewLog = new VisualReviewLog();
        visualReviewLog
                .setType(2)
                .setRelationId(params.getId())
                .setRemark(params.getReason())
                .setStatus(params.isPass() ? 1 : 2)
                .setCreateTime(LocalDateTime.now())
                .setCreateUserName(SecureUtil.getNickName())
                .setCreateUserId(SecureUtil.getUserId());
        visualReviewLogService.save(visualReviewLog);
        return true;
    }

    public boolean enableTemplate(EnableParams params) {
        visualComponentTemplateService.update(
                Wrappers.<VisualComponentTemplate>lambdaUpdate()
                        .eq(VisualComponentTemplate::getId, params.getId())
                        .set(VisualComponentTemplate::getStatus, params.getStatus())
                        .set(VisualComponentTemplate::getDisableReason, params.getDisableReason())
        );
        if (params.getStatus() == 0) {
            VisualComponentTemplate componentTemplate = visualComponentTemplateService.getById(params.getId());
            VisualReviewLog visualReviewLog = new VisualReviewLog();
            visualReviewLog.setTenantId(componentTemplate.getTenantId());
            visualReviewLog
                    .setRelationId(params.getId())
                    .setType(2)
                    .setRemark(params.getDisableReason())
                    .setStatus(3)
                    .setCreateTime(LocalDateTime.now())
                    .setCreateUserName(SecureUtil.getNickName())
                    .setCreateUserId(SecureUtil.getUserId());
            visualReviewLogService.save(visualReviewLog);
        }
        return true;
    }

    public List<VisualReviewLog> listReviewLogs(Long id) {
        return visualReviewLogService.getBaseMapper().listByRelationIdAndType(id, 2);
    }

    public List<CreateUserNameDto> getTemplateCreateUserName() {
        List<VisualComponentTemplate> list = visualComponentTemplateService.list();
        if (CollectionUtil.isEmpty(list)) return Collections.emptyList();
        List<Long> userIds = list.stream().map(VisualComponentTemplate::getCreateUser)
                .distinct().collect(Collectors.toList());
        List<UserModel> users = userClient.listByIds(userIds);
        return users.stream().map(user -> {
            CreateUserNameDto dto = new CreateUserNameDto();
            dto.setCreateUser(user.getId());
            dto.setRealName(user.getRealName());
            return dto;
        }).collect(Collectors.toList());
    }


}
