package org.springblade.visual.service;

import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springblade.core.mp.base.TenantEntity;
import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.core.tool.utils.CollectionUtil;
import org.springblade.system.client.TenantClient;
import org.springblade.system.client.UserClient;
import org.springblade.system.client.model.TenantModel;
import org.springblade.system.client.model.UserModel;
import org.springblade.visual.dto.CreateUserNameDto;
import org.springblade.visual.dto.EnableParams;
import org.springblade.visual.dto.ReviewParams;
import org.springblade.visual.dto.VisualDataPageParams;
import org.springblade.visual.entity.VisualData;
import org.springblade.visual.entity.VisualReviewLog;
import org.springblade.visual.enums.VisualTypeEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class VisualDataManageService {

    @Autowired
    private VisualDataService visualDataService;
    @Autowired
    private VisualReviewLogService visualReviewLogService;
    @Autowired
    private UserClient userClient;
    @Autowired
    private TenantClient tenantClient;

    /**
     * 大屏列表
     */
    public Page<VisualData> page(VisualDataPageParams params) {
        Page<VisualData> page = visualDataService.lambdaQuery()
                .eq(params.getStatus() != null, VisualData::getStatus, params.getStatus())
                .like(StringUtils.isNotBlank(params.getTitle()), VisualData::getTitle, params.getTitle())
                .eq(params.getTenantId() != null, TenantEntity::getTenantId, params.getTenantId())
                .eq(params.getCreateUserId() != null, VisualData::getCreateUser, params.getCreateUserId())
                .eq(params.getReviewStatus() != null, VisualData::getReviewStatus, params.getReviewStatus())
                .eq(VisualData::getVisualType, VisualTypeEnum.GENERAL)
                .eq(params.getVisualDataType() != null, VisualData::getVisualDataType, params.getVisualDataType())
                .orderByDesc(VisualData::getCreateTime)
                .page(new Page<>(params.getPage(), params.getSize()));
        List<VisualData> records = page.getRecords();
        if (page.getRecords().size() != 0) {
            List<String> tenantIds = records.stream().map(VisualData::getTenantId).collect(Collectors.toList());
            List<TenantModel> tenants = tenantClient.getTenantByTenantIds(tenantIds);
            Map<String, String> tenantMap = tenants.stream().collect(Collectors.toMap(TenantModel::getTenantId, TenantModel::getTenantName));
            for (VisualData record : records) {
                record.setTenantName(tenantMap.get(record.getTenantId()));
            }
        }
        return page;
    }

    /**
     * 审批
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean reviewVisualData(ReviewParams params) {
        visualDataService.update(
                Wrappers.<VisualData>lambdaUpdate()
                        .eq(VisualData::getId, params.getId())
                        .set(VisualData::getStatus, params.isPass() ? 1 : 0)
                        .set(VisualData::getReviewStatus, params.isPass() ? 1 : 2)
        );
        VisualReviewLog visualReviewLog = new VisualReviewLog();
        visualReviewLog.setRelationId(params.getId())
                .setRemark(params.getReason())
                .setType(1)
                .setStatus(params.isPass() ? 1 : 2)
                .setCreateTime(LocalDateTime.now())
                .setCreateUserName(SecureUtil.getNickName())
                .setCreateUserId(SecureUtil.getUserId());
        visualReviewLogService.save(visualReviewLog);
        return true;
    }

    /**
     * 禁用
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean enableVisual(EnableParams params) {
        visualDataService.update(
                Wrappers.<VisualData>lambdaUpdate()
                        .eq(VisualData::getId, params.getId())
                        .set(VisualData::getStatus, params.getStatus())
        );
        if (params.getStatus() == 0) {
            VisualData visualData = visualDataService.getById(params.getId());
            VisualReviewLog visualReviewLog = new VisualReviewLog();
            visualReviewLog.setTenantId(visualData.getTenantId());
            visualReviewLog
                    .setType(1)
                    .setRelationId(params.getId())
                    .setRemark(params.getDisableReason())
                    .setStatus(3)
                    .setCreateTime(LocalDateTime.now())
                    .setCreateUserName(SecureUtil.getNickName())
                    .setCreateUserId(SecureUtil.getUserId());
            visualReviewLogService.save(visualReviewLog);
        }
        return true;
    }

    public List<VisualReviewLog> listReviewLogs(Long id) {
        return visualReviewLogService.getBaseMapper().listByRelationIdAndType(id, 1);
    }

    public List<CreateUserNameDto> getCreateUserName() {
        List<VisualData> list = visualDataService.list();
        if (CollectionUtil.isEmpty(list)) return Collections.emptyList();
        List<Long> userIds = list.stream().map(VisualData::getCreateUser)
                .distinct().collect(Collectors.toList());
        List<UserModel> users = userClient.listByIds(userIds);
        return users.stream().map(user -> {
            CreateUserNameDto dto = new CreateUserNameDto();
            dto.setCreateUser(user.getId());
            dto.setRealName(user.getRealName());
            return dto;
        }).collect(Collectors.toList());
    }


}
