package org.springblade.visual.service;

import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springblade.core.mp.base.TenantEntity;
import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.core.tool.exception.ServiceException;
import org.springblade.core.tool.utils.CollectionUtil;
import org.springblade.system.client.TenantClient;
import org.springblade.system.client.UserClient;
import org.springblade.system.client.model.TenantModel;
import org.springblade.system.client.model.UserModel;
import org.springblade.visual.dto.CreateUserNameDto;
import org.springblade.visual.dto.EnableParams;
import org.springblade.visual.dto.ListTemplateParams;
import org.springblade.visual.dto.ReviewTemplateParams;
import org.springblade.visual.entity.VisualDataTemplate;
import org.springblade.visual.entity.VisualDataTemplateCategory;
import org.springblade.visual.entity.VisualReviewLog;
import org.springblade.visual.mapper.VisualDataTemplateCategoryMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class VisualDataTemplateManageService {

    @Autowired
    private VisualDataTemplateService visualDataTemplateService;

    @Autowired
    private VisualDataTemplateCategoryMapper visualDataTemplateCategoryMapper;
    @Autowired
    private VisualReviewLogService visualReviewLogService;
    @Autowired
    private UserClient userClient;
    @Autowired
    private TenantClient tenantClient;

    public List<VisualDataTemplateCategory> categoryList(String name) {
        return visualDataTemplateCategoryMapper
                .selectList(Wrappers.<VisualDataTemplateCategory>lambdaQuery()
                        .like(StringUtils.isNotBlank(name), VisualDataTemplateCategory::getName, name));
    }


    public List<CreateUserNameDto> getTemplateCreateUserName() {
        List<VisualDataTemplate> list = visualDataTemplateService.list();
        if (CollectionUtil.isEmpty(list)) return Collections.emptyList();
        List<Long> userIds = list.stream().map(VisualDataTemplate::getCreateUser)
                .distinct().collect(Collectors.toList());
        List<UserModel> users = userClient.listByIds(userIds);
        return users.stream().map(user -> {
            CreateUserNameDto dto = new CreateUserNameDto();
            dto.setCreateUser(user.getId());
            dto.setRealName(user.getRealName());
            return dto;
        }).collect(Collectors.toList());
    }

    public Page<VisualDataTemplate> listTemplates(ListTemplateParams params) {
        LambdaUpdateWrapper<VisualDataTemplate> wrapper = Wrappers.<VisualDataTemplate>lambdaUpdate()
                .eq(params.getCategory() != null, VisualDataTemplate::getMcpCategory, params.getCategory())
                .like(params.getName() != null, VisualDataTemplate::getName, params.getName())
                .eq(params.getCreateUser() != null, VisualDataTemplate::getCreateUser, params.getCreateUser())
                .eq(params.getTenantId() != null, TenantEntity::getTenantId, params.getTenantId())
                .eq(params.getReviewStatus() != null, VisualDataTemplate::getReviewStatus, params.getReviewStatus())
                .ne(VisualDataTemplate::getReviewStatus, 0)
                .eq(params.getStatus() != null, VisualDataTemplate::getStatus, params.getStatus())
                .between(params.getStartTime() != null && params.getEndTime() != null, VisualDataTemplate::getCreateTime, params.getStartTime()+" 00:00:00", params.getEndTime()+" 23:59:59")
                .orderByDesc(VisualDataTemplate::getCreateTime);
        Page<VisualDataTemplate> page = visualDataTemplateService.page(new Page<>(params.getPage(), params.getSize()), wrapper);
        if (page.getRecords().size() != 0) {
            List<String> tenantIds = page.getRecords().stream().map(TenantEntity::getTenantId).distinct().collect(Collectors.toList());
            List<TenantModel> tenantByTenantIds = tenantClient.getTenantByTenantIds(tenantIds);
            Map<String, String> tenantMap = tenantByTenantIds.stream().collect(Collectors.toMap(TenantModel::getTenantId, TenantModel::getTenantName));
            //获取分类信息
            Map<Integer, String> categoryMap = new HashMap<>();
            List<Integer> mcpCategoryIds = page.getRecords().stream().map(VisualDataTemplate::getMcpCategory).collect(Collectors.toList());
            if (mcpCategoryIds.size() != 0) {
                List<VisualDataTemplateCategory> visualDataTemplateCategories = visualDataTemplateCategoryMapper.selectBatchIds(mcpCategoryIds);
                categoryMap = visualDataTemplateCategories.stream().collect(Collectors.toMap(VisualDataTemplateCategory::getId, VisualDataTemplateCategory::getName));
            }
            List<Long> userIds = page.getRecords().stream().map(VisualDataTemplate::getCreateUser).collect(Collectors.toList());
            List<UserModel> userModelList = userClient.listByIds(userIds);
            Map<Long, String> accountMap = new HashMap<>();
            if (CollectionUtil.isNotEmpty(userModelList)) {
                accountMap = userModelList.stream().collect(Collectors.toMap(UserModel::getId, UserModel::getAccount));
            }
            for (VisualDataTemplate visualDataTemplate : page.getRecords()) {
                visualDataTemplate.setTenantName(tenantMap.get(visualDataTemplate.getTenantId()));
                if (categoryMap.size() != 0) {
                    visualDataTemplate.setMcpCategoryName(categoryMap.get(visualDataTemplate.getMcpCategory()));
                    visualDataTemplate.setIamAccount(accountMap.get(visualDataTemplate.getCreateUser()));
                }
            }
        }
        return page;
    }

    public boolean updateCategory(Long id, Long category) {
        return visualDataTemplateService.lambdaUpdate()
                .eq(VisualDataTemplate::getId, id)
                .set(VisualDataTemplate::getMcpCategory, category)
                .update();
    }

    @Transactional(rollbackFor = Exception.class)
    public boolean reviewTemplate(ReviewTemplateParams params) {
        visualDataTemplateService.update(
                Wrappers.<VisualDataTemplate>lambdaUpdate()
                        .eq(VisualDataTemplate::getId, params.getId())
                        .set(VisualDataTemplate::getMcpCategory, params.getCategory())
                        .set(VisualDataTemplate::getStatus, params.isPass() ? 1 : 0)
                        .set(VisualDataTemplate::getReviewStatus, params.isPass() ? 2 : 3)
        );
        VisualDataTemplate dataTemplate = visualDataTemplateService.getById(params.getId());
        VisualReviewLog visualReviewLog = new VisualReviewLog();
        visualReviewLog.setTenantId(dataTemplate.getTenantId());
        visualReviewLog
                .setType(0)
                .setRelationId(params.getId())
                .setRemark(params.getReason())
                .setStatus(params.isPass() ? 1 : 2)
                .setCreateUserName(SecureUtil.getNickName())
                .setCreateTime(LocalDateTime.now())
                .setCreateUserId(SecureUtil.getUserId());
        visualReviewLogService.save(visualReviewLog);
        return true;
    }

    public boolean enableTemplate(EnableParams params) {
        visualDataTemplateService.update(
                Wrappers.<VisualDataTemplate>lambdaUpdate()
                        .eq(VisualDataTemplate::getId, params.getId())
                        .set(VisualDataTemplate::getStatus, params.getStatus())
                        .set(VisualDataTemplate::getDisableReason, params.getDisableReason())
        );
        if (params.getStatus() == 0) {
            VisualReviewLog visualReviewLog = new VisualReviewLog();
            visualReviewLog
                    .setType(0)
                    .setRelationId(params.getId())
                    .setRemark(params.getDisableReason())
                    .setStatus(3)
                    .setCreateTime(LocalDateTime.now())
                    .setCreateUserName(SecureUtil.getNickName())
                    .setCreateUserId(SecureUtil.getUserId());
            visualReviewLogService.save(visualReviewLog);
        }
        return true;
    }

    public List<VisualReviewLog> listReviewLogs(Long id) {
        return visualReviewLogService.getBaseMapper().listByRelationIdAndType(id, 0);
    }

    public boolean deleteCategory(Long id) {
        //判断是否有子分类
        List<VisualDataTemplateCategory> list = visualDataTemplateCategoryMapper.selectList(Wrappers.<VisualDataTemplateCategory>lambdaQuery().eq(VisualDataTemplateCategory::getParentId, id));
        if (list.size() > 0) throw new ServiceException("请先删除子分类");
        //判断目录下面是否有模板
        List<VisualDataTemplate> templateList = visualDataTemplateService.lambdaQuery().eq(VisualDataTemplate::getMcpCategory, id).list();
        if (cn.hutool.core.collection.CollectionUtil.isNotEmpty(templateList)) throw new ServiceException("目录下存在模板");
        visualDataTemplateCategoryMapper.deleteById(id);
        return true;
    }

}
