package org.springblade.visual.service;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.lang.UUID;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.exception.ServiceException;
import org.springblade.visual.dto.ListTemplateParams;
import org.springblade.visual.dto.SubmitTemplateReviewParams;
import org.springblade.visual.entity.VisualData;
import org.springblade.visual.entity.VisualDataTemplate;
import org.springblade.visual.entity.VisualDataTemplateCategory;
import org.springblade.visual.entity.VisualReviewLog;
import org.springblade.visual.enums.VisualTypeEnum;
import org.springblade.visual.mapper.VisualDataTemplateCategoryMapper;
import org.springblade.visual.mapper.VisualDataTemplateMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Lazy;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RequestBody;

import java.time.LocalDateTime;
import java.util.*;

@Service
public class VisualDataTemplateService extends ServiceImpl<VisualDataTemplateMapper, VisualDataTemplate> {

    @Autowired
    private VisualDataTemplateCategoryMapper visualDataTemplateCategoryMapper;
    @Autowired
    @Lazy
    private VisualDataTemplateManageService visualDataTemplateManageService;
    @Autowired
    private VisualReviewLogService visualReviewLogService;
    @Autowired
    private VisualDataService visualDataService;


    public List<VisualDataTemplateCategory> categoryList(String name) {
        List<VisualDataTemplateCategory> categoryList = visualDataTemplateCategoryMapper
                .selectList(Wrappers.<VisualDataTemplateCategory>lambdaQuery()
                        .like(StringUtils.isNotBlank(name), VisualDataTemplateCategory::getName, name));
        return categoryList;
    }

    public boolean submitCategory(VisualDataTemplateCategory category) {
        if (category.getId() == null) {
            category.setCreateUser(SecureUtil.getUserId());
            visualDataTemplateCategoryMapper.insert(category);
        } else {
            category.setUpdateUser(SecureUtil.getUserId());
            visualDataTemplateCategoryMapper.updateById(category);
        }
        return true;
    }

    public boolean deleteCategory(Long id) {
        //判断是否有子分类
        List<VisualDataTemplateCategory> list = visualDataTemplateCategoryMapper.selectList(Wrappers.<VisualDataTemplateCategory>lambdaQuery().eq(VisualDataTemplateCategory::getParentId, id));
        if (list.size() > 0) {
            throw new ServiceException("请先删除子分类");
        }
        //判断目录下面是否有模板
        List<VisualDataTemplate> templateList = this.lambdaQuery().eq(VisualDataTemplate::getCategory, id).list();
        if (CollectionUtil.isNotEmpty(templateList)) {
            throw new ServiceException("目录下存在模板");
        }
        visualDataTemplateCategoryMapper.deleteById(id);
        return true;
    }


    public Page<VisualDataTemplate> listTemplates(ListTemplateParams params) {
        LambdaUpdateWrapper<VisualDataTemplate> wrapper = Wrappers.<VisualDataTemplate>lambdaUpdate()
                .eq(params.getCategory() != null, VisualDataTemplate::getCategory, params.getCategory())
                .like(params.getName() != null, VisualDataTemplate::getName, params.getName())
                .eq(params.getCreateUser() != null, VisualDataTemplate::getCreateUser, params.getCreateUser())
                .eq(params.getReviewStatus() != null, VisualDataTemplate::getReviewStatus, params.getReviewStatus())
                .eq(params.getStatus() != null, VisualDataTemplate::getStatus, params.getStatus())
                .between(params.getStartTime() != null && params.getEndTime() != null, VisualDataTemplate::getCreateTime, params.getStartTime(), params.getEndTime())
                .orderByDesc(VisualDataTemplate::getCreateTime);
        Page<VisualDataTemplate> page = this.page(new Page<>(params.getPage(), params.getSize()), wrapper);
        return page;
    }

    public Boolean removeTemplate(Long id) {
        return this.removeById(id);
    }

    public Boolean checkOperationPermissions(Long id) {
        VisualDataTemplate dataTemplate = this.getById(id);
        //等于0代表只有预览权限 校验是否是创建人
        if (dataTemplate.getOperationPermissions() == 0) {
            if (dataTemplate.getCreateUser() != AuthUtil.getUserId()) {
                throw new ServiceException("该模板只有查看权限");
            }
        }
        return true;
    }


    @Transactional(rollbackFor = Exception.class)
    public Boolean copyTemplate(Long id, BladeUser bladeUser) {
        VisualDataTemplate dataTemplate = this.getById(id);
        //等于0代表只有预览权限 校验是否是创建人
        if (dataTemplate.getOperationPermissions() == 0) {
            if (dataTemplate.getCreateUser() != AuthUtil.getUserId()) {
                throw new ServiceException("该模板只有查看权限");
            }
        }
        dataTemplate.setName("[副本]" + dataTemplate.getName()).setId(null);
        List<Integer> visualIdList = visualDataService
                .doCopy(CollectionUtil.newHashSet(dataTemplate.getVisualId()),
                        null, null, null,
                        VisualTypeEnum.TEMPLATE, bladeUser, null, null);
        dataTemplate.setVisualId(visualIdList.get(0))
                .setStatus(0)
                .setCreateUser(AuthUtil.getUserId())
                .setCreateUserName(AuthUtil.getNickName())
                .setReviewStatus(0);
        dataTemplate.setTenantId(AuthUtil.getTenantId());
        this.save(dataTemplate);
        return true;
    }

    public List<VisualReviewLog> listReviewLogs(Long id) {
        return visualReviewLogService.getBaseMapper().listByRelationIdAndType(id, 0);
    }

    @Transactional(rollbackFor = Exception.class)
    public boolean submitReview(@RequestBody SubmitTemplateReviewParams params) {
        this.lambdaUpdate().eq(VisualDataTemplate::getId, params.getTemplateId())
                .set(VisualDataTemplate::getOperationPermissions, params.getOperationPermissions())
                .update();
        VisualReviewLog visualReviewLog = new VisualReviewLog();
        visualReviewLog.setRemark(params.getReason())
                .setStatus(0)
                .setType(0)
                .setRelationId(params.getTemplateId())
                .setCreateTime(LocalDateTime.now())
                .setCreateUserName(AuthUtil.getNickName())
                .setCreateUserId(SecureUtil.getUserId());
        visualReviewLogService.save(visualReviewLog);
        //修改审核状态
        this.lambdaUpdate()
                .eq(VisualDataTemplate::getId, params.getTemplateId())
                .set(VisualDataTemplate::getReviewStatus, 1)
                .update();
        return true;
    }


    /**
     * 修改大屏模板基本信息
     *
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean updateTemplateBasicInfo(VisualDataTemplate template) {
        this.updateById(template);
        visualDataService.lambdaUpdate()
                .eq(VisualData::getId, template.getVisualId())
                .set(VisualData::getCategory, template.getCategory())
                .update();
        return true;
    }

    /**
     * 所有审核通过的模板
     */
    public Page<VisualDataTemplate> allReviewedTemplate(Integer page, Integer size, String name, String category) {
        return this.getBaseMapper().allReviewedTemplate(new Page(page, size), name, category);
    }

    /**
     * 所有目录
     */
    public List<VisualDataTemplateCategory> allMcpCategory(String name) {
        List<VisualDataTemplateCategory> categoryList = visualDataTemplateCategoryMapper.allMcpCategory(name);
        return Optional.ofNullable(categoryList).orElse(new ArrayList<>());
    }

    /**
     * 大屏模版计数
     *
     * @param templateId
     */
    @Async
    public void addTemplateUseTimes(Long templateId) {
        baseMapper.addUseTimes(templateId);
    }

}
