/*
 * Copyright (c) 2018-2028, aokunsang All rights reserved.
 * 我问青山何时老，青山问我几时闲，
 * 不是闲人闲不得，能闲必非等闲人。
 */
package org.springblade.visual.service.component;

import java.util.List;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;

import org.springblade.core.tool.exception.ServiceException;
import org.springblade.cube.enums.DimensionTypeEnum;
import org.springblade.datasource.context.DialectContext;
import org.springblade.datasource.enums.DsTypeEnum;
import org.springblade.datasource.enums.FunctionMethodEnum;
import org.springblade.datasource.enums.ViewColumnTypeEnum;
import org.springblade.visual.dto.ComponentColumn;
import org.springblade.visual.dto.ComponentColumnCompare;
import org.springblade.visual.enums.CompareTypeEnum;
import org.springblade.visual.enums.ComponentTypeEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * 组件同环比字段设置
 *
 * @author wookvn
 * @datetime 2024-10-16 14:41:48
 */
@Slf4j
@Component
public class VisualComponentCompareService {

    @Autowired
    private DialectContext dialectContext;

    /**
     * 获取上期数据的SQL
     */
    final static String LAG_SQL_FORMAT = "LAG(%s, %s) OVER (%sORDER BY %s)";
    public final static String LAG_SYMBOL = "_pre";

    /**
     * <p>抽取并处理同环比设置字段</p>
     * @author wookvn
     * @datetime 2024-10-16 15:28:42
     * @param columnList: 维度指标字段列表
     * @param compType: 组件类型
     * @param dsType：数据库类型
     **/
    public void preCompareHandle(List<ComponentColumn> columnList, ComponentTypeEnum compType, DsTypeEnum dsType) {
        //获取非时间类型的维度字段列表，作为partition by字段
        Set<String> partitionByList = columnList.stream().filter(item -> item.getDimensionType() == DimensionTypeEnum.WEIDU && item.getViewDataType() != ViewColumnTypeEnum.DATETIME)
                .map(ComponentColumn::getColumnSql).collect(Collectors.toSet());
        String partitionBySql = CollectionUtil.isNotEmpty(partitionByList) ? ("PARTITION BY " + String.join(",", partitionByList) + " ") : "";
        //获取需要处理同环比的字段
        List<ComponentColumn> compareColumnList = columnList.stream().filter(item -> item.getDimensionType() == DimensionTypeEnum.ZHIBIAO && item.getCompare() != null && StrUtil.isNotBlank(item.getCompare().getDateFieldUniqId()))
                .collect(Collectors.toList());
        if(CollectionUtil.isEmpty(compareColumnList)) {
            return;
        }
        //循环遍历开始处理这些字段，并且重新添加到columnList中
        for(ComponentColumn compareColumn : compareColumnList) {
            ComponentColumnCompare columnCompare = compareColumn.getCompare();
            ComponentColumn dateField = columnList.stream().filter(item -> columnCompare.getDateFieldUniqId().equals(item.getUniqId())).findFirst().orElse(null);
            if(dateField == null) {
                throw new ServiceException("同环比设置参数有错误|日期字段不存在");
            }
            FunctionMethodEnum functionMethod = dateField.getFunctionMethod();
            CompareTypeEnum compareType = columnCompare.getCompareType();
            if(functionMethod == null) {
                throw new ServiceException("同环比设置参数有错误|日期字段的functionMethod必须设置");
            }
            if(compareType == null) {
                throw new ServiceException("同环比设置参数有错误|对比类型必须设置");
            }
            if(!compareType.supports(functionMethod)) {
                throw new ServiceException(String.format("%s不支持该类型的日期字段", compareType.getMessage()));
            }
            //本期SQL片段
            String curPeriodSql = compareColumn.getColumnSql();
            //聚合函数 [且计算字段是不包含聚合函数的]
            if (compareColumn.getAggregationType() != null && compType != ComponentTypeEnum.DETAIL && Optional.<Integer>ofNullable(compareColumn.getVirtualAggregationStatus()).orElse(0) == 0) {
                curPeriodSql = String.format(compareColumn.getAggregationType().getMethod(), curPeriodSql);
            }
            String orderBySql = dialectContext.getDialect(dsType).formatMethodFunction(functionMethod, dateField.getColumnSql(), dateField.getColumnSql());
            //间隔多久
            int periodValue = 1;
            if(functionMethod == FunctionMethodEnum.EXTRACT_QUARTER && compareType == CompareTypeEnum.YEAR_SELF) {
                periodValue = 4;
            } else if (functionMethod == FunctionMethodEnum.EXTRACT_WEEK && compareType == CompareTypeEnum.YEAR_SELF) {
                periodValue = 52;
            } else if (functionMethod == FunctionMethodEnum.EXTRACT_MONTH && compareType == CompareTypeEnum.YEAR_SELF) {
                periodValue = 12;
            } else if (functionMethod == FunctionMethodEnum.EXTRACT_DAY && compareType == CompareTypeEnum.WEEK_SELF) {
                periodValue = 7;
            } else if (functionMethod == FunctionMethodEnum.EXTRACT_DAY && compareType == CompareTypeEnum.MONTH_SELF){
                periodValue = 30;
            } else if (functionMethod == FunctionMethodEnum.EXTRACT_DAY && compareType == CompareTypeEnum.YEAR_SELF){
                periodValue = 365;
            }
            //LAG(SUM(`price`), 1) OVER (PARTITION BY `region` ORDER BY DATE_FORMAT(`order_time`,'%%Y%%m'))
            String prePeriodSql = String.format(LAG_SQL_FORMAT, curPeriodSql, periodValue, partitionBySql, orderBySql);
            //上一期的字段列
            ComponentColumn prePeriodColumn = new ComponentColumn();
            prePeriodColumn.setColumnSql(prePeriodSql);
            prePeriodColumn.setAggregationType(null);
            prePeriodColumn.setDimensionType(DimensionTypeEnum.ZHIBIAO);
            prePeriodColumn.setColumnAliasName(compareColumn.getColumnAliasName() + LAG_SYMBOL);
            prePeriodColumn.setColumnTitle(compareColumn.getColumnTitle());
            columnList.add(prePeriodColumn);
        }
    }
}
