package org.springblade.visual.service.component;

import static org.springblade.core.secure.utils.AuthUtil.getUser;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

import org.springblade.core.tool.api.R;
import org.springblade.core.tool.exception.ServiceException;
import org.springblade.cube.dto.PreviewResult;
import org.springblade.cube.enums.DimensionTypeEnum;
import org.springblade.cube.model.CubeData;
import org.springblade.cube.service.CubeDataService;
import org.springblade.cube.service.DynamicTableService;
import org.springblade.properties.ComponentProperties;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.service.IReportDatasourceService;
import org.springblade.visual.dto.ComponentColumn;
import org.springblade.visual.dto.ComponentDto;
import org.springblade.visual.dto.QuerySqlParam;
import org.springblade.visual.entity.VisualComponent;
import org.springblade.visual.service.QuerySqlService;
import org.springblade.visual.service.VisualComponentService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSON;
import com.microsoft.sqlserver.jdbc.StringUtils;

import cn.hutool.core.lang.Tuple;

/**
 * <p>直方图</p>
 * @author  wookvn
 * @datetime 2024-10-12 12:28:41
 */
@Service
public class VisualComponentHistogramService {

    @Autowired
    private VisualComponentService visualComponentService;

    @Autowired
    private DynamicTableService dynamicTableService;

    @Autowired
    private QuerySqlService querySqlService;

    @Autowired
    private CubeDataService cubeDataService;

    @Autowired
    private IReportDatasourceService reportDatasourceService;

    @Autowired
    private ComponentProperties componentProperties;

    /**
     * 最大值最小值
     */
    private List<BigDecimal> getMinMax(ComponentDto componentDto, CubeData cubeData, ReportDatasource reportDatasource) {
        List<ComponentColumn> columnList = JSON.parseArray(componentDto.getColumnList(), ComponentColumn.class);
        ComponentColumn zbColumn = columnList.stream().filter(column -> column.getDimensionType() == DimensionTypeEnum.ZHIBIAO).findFirst().get();
        String tableAliasName = zbColumn.getTableAliasName();
        String columnName = zbColumn.getColumnName();
        //构造全局变量
        Map<String, List<Object>> extraParams = querySqlService.buildGlobalParams(componentDto.getExtraParams(), getUser());
        //获取最大值 最小值
        String minMaxSqlSelect = String.format("select min(%s) histogram_min,max(%s) histogram_max from %s", tableAliasName + "." + columnName, tableAliasName + "." + columnName, cubeData.getQuerySql());
        Tuple whereHavingSql = querySqlService.parseWhereHavingSql(cubeData, reportDatasource.getDsTypeSyl(), componentDto, getUser(), extraParams);
        String whereSql = whereHavingSql.get(0), havingSql = whereHavingSql.get(1);
        if (!StringUtils.isEmpty(whereSql)) {
            minMaxSqlSelect = minMaxSqlSelect + " where " + whereSql;
        }
        if (!StringUtils.isEmpty(havingSql)) {
            minMaxSqlSelect = minMaxSqlSelect + " having " + havingSql;
        }

        List<Map<String, Object>> minMaxResult = dynamicTableService.selectList(reportDatasource, minMaxSqlSelect);
        //四舍五入取整
        Map<String, Object> map = minMaxResult.get(0);
        List<BigDecimal> minMaxDecimals = new ArrayList<>();
        minMaxDecimals.add(new BigDecimal(map.get("histogram_min").toString()));
        minMaxDecimals.add(new BigDecimal(map.get("histogram_max").toString()));
        return minMaxDecimals;
    }

    /**
     * 添加直方图 获取区间数据的脚本
     */
    private List<BigDecimal> addCaseWhenColumn(List<BigDecimal> minMaxDecimals, ComponentDto componentDto) {
        Integer histogramCountNew = Optional.ofNullable(componentDto.getHistogramCount()).orElse(componentProperties.getHistogramCount());
        Integer histogramDecimalPlacesNew = Optional.ofNullable(componentDto.getHistogramDecimalPlaces()).orElse(componentProperties.getHistogramDecimalPlaces());
        String caseWhen = "SUM(CASE WHEN %s >= %s AND %s < %s THEN 1 ELSE 0 END)";
        List<ComponentColumn> columnList = JSON.parseArray(componentDto.getColumnList(), ComponentColumn.class);
        ComponentColumn zbColumn = columnList.stream().filter(column -> column.getDimensionType() == DimensionTypeEnum.ZHIBIAO).findFirst().get();
        BigDecimal min = minMaxDecimals.get(0).setScale(histogramDecimalPlacesNew, RoundingMode.DOWN);
        BigDecimal max = minMaxDecimals.get(1).setScale(histogramDecimalPlacesNew, RoundingMode.UP);
        //计算区间宽度
        BigDecimal width = max.subtract(min).divide(new BigDecimal(histogramCountNew), histogramDecimalPlacesNew, RoundingMode.UP);
        List<BigDecimal> bigDecimals = new ArrayList<>();
        for (int i = 0; i < histogramCountNew; i++) {
            BigDecimal start = min.add(width.multiply(new BigDecimal(i)));
            if (i==0){
                bigDecimals.add(start);
            }
            BigDecimal end = min.add(width.multiply(new BigDecimal(i + 1)));
            bigDecimals.add(end);
            String sql = String.format(caseWhen,
                    zbColumn.getTableAliasName() + "." + zbColumn.getColumnName(),
                    start,
                    zbColumn.getTableAliasName() + "." + zbColumn.getColumnName(),
                    end);
            ComponentColumn componentColumn = new ComponentColumn();
            componentColumn.setColumnSql(sql);
            componentColumn.setColumnAliasName("histogram_" + start.toString().replace(".","_"));
            componentColumn.setColumnTitle("histogram_" + start.toString().replace(".","_"));
            columnList.add(componentColumn);
        }
        componentDto.setColumnList(JSON.toJSONString(columnList));
        return bigDecimals;
    }

    /**
     * 直方图预览数据后置处理
     *
     * @param componentDto
     * @return
     * @throws Exception
     */
    private void postHistogramData(ComponentDto componentDto, List<Map<String, Object>> results, List<BigDecimal> bigDecimals, Map<String, String> fieldDict) {
        List<ComponentColumn> columnList = JSON.parseArray(componentDto.getColumnList(), ComponentColumn.class);
        List<String> histogramColumnList = columnList.stream().filter(componentColumn -> componentColumn.getColumnAliasName() != null && componentColumn.getColumnAliasName().contains("histogram_")).map(ComponentColumn::getColumnAliasName).collect(Collectors.toList());
        for (Map<String, Object> result : results) {
            List<List<BigDecimal>> list = new ArrayList<>();
            for (int i = 0; i < histogramColumnList.size(); i++) {
                List<BigDecimal> list1 = new ArrayList<>();
                String s = histogramColumnList.get(i);
                BigDecimal count = new BigDecimal(result.get(s).toString());
                list1.add(bigDecimals.get(i));
                list1.add(bigDecimals.get(i+1));
                list1.add(count);
                list.add(list1);
            }
            result.put("histogram", list);
            //删除histogram_开头的数据
            histogramColumnList.stream().forEach(result::remove);
        }
        //删除histogram_开头的字典key
        histogramColumnList.stream().forEach(fieldDict::remove);
    }

    public Tuple histogramPreview(String value) throws Exception {
        ComponentDto componentDto = visualComponentService.previewCheck(value, false, componentDtoPram -> {
            List<ComponentColumn> columnList = JSON.parseArray(componentDtoPram.getColumnList(), ComponentColumn.class);
            List<ComponentColumn> zbColumns = columnList.stream().filter(column -> column.getDimensionType() == DimensionTypeEnum.ZHIBIAO).collect(Collectors.toList());
            if (zbColumns.size()>1){
                throw new ServiceException("直方图只能有一个指标");
            }
        });
        VisualComponent component = componentDto.getComponent();
        CubeData cubeData = cubeDataService.getById(component.getCubeDataId());
        //2.查询数据源
        ReportDatasource reportDatasource1 = reportDatasourceService.getById(cubeData.getDatasourceId());
        if (reportDatasource1 == null) {
            throw new ServiceException("数据源不存在");
        }
        List<BigDecimal> minMax = getMinMax(componentDto, cubeData, reportDatasource1);
        List<BigDecimal> bigDecimals = addCaseWhenColumn(minMax, componentDto);
        R<QuerySqlParam> r = querySqlService.parseQuerySql(component.getCubeDataId(), componentDto, getUser());
        if (!r.isSuccess()) {
            throw new ServiceException(r.getMsg());
        }
        QuerySqlParam querySqlParam = r.getData();
        ReportDatasource reportDatasource = querySqlParam.getReportDatasource();
        String querySql = querySqlParam.getQuerySql();
        List<ComponentColumn> selectColumnList = querySqlParam.getSelectColumnList();
        Map<String, String> fieldDict = selectColumnList.stream().collect(Collectors.toMap(ComponentColumn::getColumnAliasName, ComponentColumn::getColumnTitle));
        List<Map<String, Object>> results = dynamicTableService.selectList(reportDatasource, querySql);
        //预览数据后置处理
//        visualComponentService.postPreviewData(fieldDict, results, selectColumnList, querySqlParam.getExcludeColumnIdList(), querySqlParam);
        String cubeName = querySqlParam.getCubeData() != null ? querySqlParam.getCubeData().getCubeName() : null;
        postHistogramData(componentDto, results, bigDecimals, fieldDict);
        return new Tuple(new PreviewResult(results, fieldDict, cubeName), querySql);
    }
}
